﻿using BenchmarkDotNet.Attributes;
using BenchmarkDotNet.Running;

namespace StructClassBenchmark
{
    public struct StructDouble
    {
        public double Value { get; set; }

        public StructDouble(double value) => Value = value;

        public static StructDouble operator +(StructDouble left, StructDouble right) => 
            new StructDouble(left.Value + right.Value);
    }

    public class ClassDouble
    {
        public double Value { get; set; }

        public ClassDouble(double value) => Value = value;

        public static ClassDouble operator +(ClassDouble left, ClassDouble right) =>
            new ClassDouble(left.Value + right.Value);
    }

    [MemoryDiagnoser]
    public class StructClassBenchmark
    {
        private const int N = 100_000;

        [Benchmark]
        public ClassDouble Class()
        {
            ClassDouble sum = new ClassDouble(0);

            for (int i = 0; i < N; i++)
            {
                sum += new ClassDouble(1);
            }

            return sum;
        }

        [Benchmark(Baseline = true)]
        public StructDouble Struct()
        {
            StructDouble sum = new StructDouble(0);

            for (int i = 0; i < N; i++)
            {
                sum += new StructDouble(1);
            }

            return sum;
        }

		[Benchmark]
		public double PlainDouble() {
			double sum = 0.0;

			for (int i = 0; i < N; i++) {
				sum += 1.0;
			}

			return sum;
		}
	}

    class Program
    {
        static void Main(string[] args)
        {
            BenchmarkRunner.Run<StructClassBenchmark>();
        }
    }
}
